<?php
/*
Plugin Name: BBToolbar
Version: 1.2
Plugin URI: http://www.vdfn.altervista.org/redirect/plugin_bbtoolbar.html
Description: This is an alternative toolbar for BBCode.
Author: Piero VDFN
Author URI: http://www.vdfn.altervista.org/
*/

/**
 * This is the cache file of the plugin
 */
define('BBTOOLBAR_CACHE', CACHE_DIR.'bbtoolbar.js');

/**
 * This is the base path of the cache of the emoticons.
 */
define('BBTOOLBAR_EMCACHE', CACHE_DIR.'bbtoolbar_e%s.tmp');

/**
 * This is a debug option: always force the compilation of Javascript.
 */
define('BBTOOLBAR_FORCECOMPILE', false);

/**
 * This is the main class of the plugin.
 */
class plugin_bbtoolbar {
	/**
	 * This is the stored configuration.
	 *
	 * @var array
	 */
	var $conf=array();

	/**
	 * Should also create a placeolder for the toolbar?
	 *
	 * @var boolean
	 */
	var $placeholder=false;

	/**
	 * The smiliesSearch variable is used for emoticons replacing.
	 * Original code used it as a static variable but I prefer
	 * using it as a variable of the instance.
	 *
	 * @var array
	 */
	var $smiliesSearch=null;

	/**
	 * The smiliesReplace variable is used for emoticons replacing.
	 * Original code used it as a static variable but I prefer
	 * using it as a variable of the instance.
	 *
	 * @var array
	 */
	var $smiliesReplace=null;

	/**
	 * This variable is used to have two spaces from the same smiley
	 * instead of one. These are the texts to search.
	 *
	 * @var array
	 */
	var $smiliesDoubleS=null;

	/**
	 * This variable is used to have two spaces from the same smiley
	 * instead of one. These are the texts to replace.
	 *
	 * @var array
	 */
	var $smiliesDoubleR=null;

	/**
	 * This is the constructor of the class. 
	 * It manages hooks.
	 */
	function plugin_bbtoolbar() {
		// Get the configuration
		$conf=$this->getConf();
		// For the comments
		if(!defined('MOD_ADMIN_PANEL')) {
			add_action('wp_footer', array(&$this, 'footer'));
		} elseif($conf['enEn'] || $conf['emEnTB']) {
			$this->placeholder=true;
			add_filter('editor_toolbar', array(&$this, 'footer'));
		}

		// Get the "compiled" code
		add_action('init', array(&$this, 'init'));

		// Replace the similies
		if($conf['emEnP']) {
			add_filter('the_content', array(&$this, 'emoticonsReplace'), 5);
		}
		if($conf['emCoP']) {
			add_filter('comment_text', array(&$this, 'emoticonsReplace'), 5);
		}

		// The stylesheet
		add_action('wp_head', array(&$this, 'head'), 8);
	}

	/**
	 * This function prints the javascript of the plugin.
	 * The function is called by the hook wp_foot.
	 */
	function footer() {
		if($this->placeholder) {
			$place="<div class=\"bbtoolbar_placeholder\"></div>\n";
		} else {
			$place='';
		}

		//$url=plugin_geturl('bbtoolbar').'res/bbtoolbar.js';
		$url=BLOG_BASEURL.'?bbtoolbar';
		$script='<script type="text/javascript" src="'.$url.'"></script>';

		echo "<!-- PLUGIN BBTOOLBAR -->\n{$place}{$script}\n<!-- END OF BBTOOLBAR -->\n";
	}

	/**
	 * This function adds the stylesheet of the plugin.
	 */
	function head() {
		$url=plugin_geturl('bbtoolbar').'res/bbtoolbar.css';
		echo "\n".'<link rel="stylesheet" type="text/css" href="'.$url.'" />'."\n";

		if(defined('MOD_ADMIN_PANEL') && @constant('ADMIN_PANEL_ACTION')=='write') {
			$url=BLOG_BASEURL.'?bbtoolbar=files';
			echo '<script type="text/javascript" src="'.$url.'"></script>'."\n";
		}
	}

	/**
	 * This function is executed at the init hook.
	 * It checks if it has to print the Javascript.
	 *
	 * It's also used to make the list of attachments.
	 */
	function init() {
		if(!isset($_GET['bbtoolbar'])) {
			return;
		}

		if($_GET['bbtoolbar']=='files' && user_loggedin()) {
			foreach(array('attachs', 'images') as $type) {
				$dir=constant(strtoupper($type).'_DIR');

				$lister=new fs_pathlister($dir);
				$list=$lister->getList();
				sort($list);
				$ignore=strlen($dir)+1;

				header('Content-type: application/x-javascript');
				echo "bbt{$type}=[\n";
				foreach((array)$list as $file) {
					echo "\t'".addslashes(substr($file, $ignore))."',\n";
				}
				echo "];\n";
			}
			die;
		}

		if(!file_exists(BBTOOLBAR_CACHE) || BBTOOLBAR_FORCECOMPILE) {
			if(!$this->compile()) {
				return;
			}
		}

		// Send some headers
		header('Pragma: Private');
		header('Cache-control: Private');

		// Last modified from http://ontosys.com/php/cache.html
		$mtime=filemtime(BBTOOLBAR_CACHE);
		if($mtime!==false) {
			$gmtime_mod=gmdate('D, d M Y H:i:s', $mtime).' GMT';

			if(@trim($_SERVER['HTTP_IF_MODIFIED_SINCE'])==$gmtime_mod) {
				header('HTTP/1.0 304 Not Modified');
				die;
			}

			header('Last-Modified: '.$gmtime_mod);
		}

		// Send the javascript
		header('Content-type: application/x-javascript');
		echo io_load_file(BBTOOLBAR_CACHE);
		die;
	}

	/**
	 * This function, called by hooks, replaces the emoticons with
	 * the images.
	 *
	 * @param string $text: The content to parse
	 * @return string: $text edited
	 *
	 * @author DMKE (For the replacement part)
	 * @author Piero VDFN (For the integration with the plugin)
	 * @link http://www.informatik.uni-bremen.de/~dmke/2008/10/08/flatpress-plugin-smilies/comments/
	 */
	function emoticonsReplace($text) {
		$delta=microtime(true);


		// My part
		$conf=$this->getConf();
		$set=$conf['emSet'];
		$replacements=$this->parseEmoticons($set);
		$baseurl=plugin_geturl('bbtoolbar').'e_'.$set.'/';

		// The DMKE part
		if(is_null($this->smiliesSearch) && is_null($this->smiliesReplace)) {
			$smiliesSearch=array();
			$smiliesReplace=array();
			$smiliesDoubleS=array();
			$smiliesDoubleR=array();

			foreach($replacements as $smiley=>$img) {
				$smileyReplace=wp_specialchars($smiley);
				$smiliesDoubleS[]=$smileyReplace.' '.$smileyReplace;
				$smiliesDoubleR[]=$smileyReplace.'  '.$smileyReplace;

				//We're looking for smilies encapsulated by spaces or at the end or beginning of a line.
				$smiliesSearch[]='/(\s|^)'.preg_quote($smileyReplace, '/').'(\s|$)/';

				// We'll use the smiley also for the alt attribute: it's better escape it
				$smileyMasked=wp_specialchars($smiley, true);
				$smileyMasked="alt=\"{$smileyMasked}\" title=\"{$smileyMasked}\"";
				$smiliesReplace[]=' <img src="'.$baseurl.$img.'" '.$smileyMasked.' class="wp-smiley" /> ';
			}

			$this->smiliesSearch=$smiliesSearch;
			$this->smiliesReplace=$smiliesReplace;
			$this->smiliesDoubleS=$smiliesDoubleS;
			$this->smiliesDoubleR=$smiliesDoubleR;
		} else {
			$smiliesSearch=$this->smiliesSearch;
			$smiliesReplace=$this->smiliesReplace;
			$smiliesDoubleS=$this->smiliesDoubleS;
			$smiliesDoubleR=$this->smiliesDoubleR;
		}

		$output='';
		// We won't replace the Smileys inside tags
		$textarr=preg_split('/(<.*>)/U', $text, -1, PREG_SPLIT_DELIM_CAPTURE);
		$count=count($textarr);

		for($i=0; $i<$count; $i++) {
			$content=$textarr[$i];
			$content=str_replace($smiliesDoubleS, $smiliesDoubleR, $content);
			$content=preg_replace($smiliesSearch, $smiliesReplace, $content);
			$output.=$content;
		}

		return $output;
	}

	/**
	 * This function return the array with the configuration of the plugin.
	 *
	 * @param boolean $force: Force to reload config?
	 * @param boolean $raw: Don't adjust enCo auto? 
	 * @return array: The configuration
	 */
	function getConf($force=false, $raw=false) {
		if(!$force && !empty($this->conf)) {
			return $this->conf;
		}

		$default=array(
			'enEn'=>true,
			'enCo'=>'auto',
			'emEnTB'=>true,
			'emCoTB'=>true,
			'emSet'=>'lantaca',
			'emEnP'=>true,
			'emCoP'=>true,
		);

		$conf=plugin_getoptions('bbtoolbar');

		$this->conf=array_merge($default, (array)$conf);

		if($this->conf['enCo']=='auto' && !$raw) {
			$this->conf['enCo']=BBCODE_ENABLE_COMMENTS;
		}

		return $this->conf;
	}

	/**
	 * This function compile the javascript: it's used for
	 * lang replace etc...
	 *
	 * @return boolean
	 */
	function compile() {
		// Disable error reporting
		@$level=error_reporting(0);

		// Load lang
		global $lang;
		lang_load('plugin:bbtoolbar');
		$plang=$lang['plugin']['bbtoolbar'];

		// Load JS and replace lang strings
		$content=io_load_file(dirname(__FILE__).'/res/bbtoolbar.js');
		$content=preg_replace('/~~(\w*)~~/eim', "addslashes(\$plang['$1'])", $content);

		// Load the user configuration
		$conf=$this->getConf();
		$confOptions=array(
			'\'enEn\' : '.($conf['enEn'] ? 'true' : 'false'),
			'\'enCo\' : '.($conf['enCo'] ? 'true' : 'false'),
			'\'emEnTB\' : '.($conf['emEnTB'] ? 'true' : 'false'),
			'\'emCoTB\' : '.($conf['emCoTB'] ? 'true' : 'false'),
			'\'emSet\' : \''.$conf['emSet'].'\'',
			'\'blog_url\' : \''.BLOG_BASEURL.'\'',
			'\'plugin_url\' : \''.plugin_geturl('bbtoolbar').'\'',
			'\'tag\' : '.(class_exists('plugin_tag') ? 'true' : 'false'),  
		);
		$confUser="// User configuration\nbbtConfiguration={\n\t";
		$confUser.=implode(",\n\t", $confOptions);
		$confUser.="\n};\n// EOF User configuration";

		// Load the emoticons
		$set=$this->parseEmoticons($conf['emSet']);
		$set=array_reverse($set);
		$set=array_flip($set);
		$set=array_reverse($set);
		$confUser.="\n// Emoticons\nbbtEmoticons={\n";		
		foreach($set as $image=>$symbol) {
			$symbol=addslashes($symbol);
			$confUser.="\t'{$symbol}' : '{$image}',\n";
		}
		$confUser.="};\n// EOF Emoticons";

		// Replace the default configuration with the user configuration
		$confBegin=strpos($content, '//## DEFAULT CONFIG');
		if($confBegin==0) {
			$pre='';
		} else {
			$pre=substr($content, 0, $confBegin);
		}
		$confEnd=strpos($content, '//## EOF DEFAULT CONFIG')+23;
		$post=substr($content, $confEnd);
		$content=$pre.$confUser.$post;

		// Save the compiled JS
		$return=io_write_file(BBTOOLBAR_CACHE, $content);

		// Re-enable error-reporting
		error_reporting($level);

		return $return;
	}

	/**
	 * This function parses an emoticons set.
	 *
	 * @param string $set: The emoticons set
	 * @param boolean $cache: Use a cache mechanism
	 * @return array: The emoticons replacements.
	 */
	function parseEmoticons($set, $cache=true) {
		if($cache) {
			$cachefile=sprintf(BBTOOLBAR_EMCACHE, $set);
			if(file_exists($cachefile)) {
				include $cachefile;
				if(!empty($emoticons)) {
					return $emoticons;
				}
			}
		}

		$dir=dirname(__FILE__)."/e_{$set}/";

		if(!is_dir($dir) || !file_exists($file=$dir.'set.txt')) {
			return false;
		}

		$set=trim(io_load_file($file));
		$set=(array)explode("\n", $set);

		$return=array();

		foreach($set as $line) {
			$line=trim($line);
			if(substr($line, 0, 1)=='#') {
				// A comment, ignore the line
				continue;
			}

			// Get the replacements and the image
			$parts=explode("\t", $line);

			if(empty($parts) || count($parts)<2) {
				continue;
			}

			$replacements=array_filter(explode(' ', $parts[0]));
			$images=array_pad(array(), count($replacements), trim($parts[1]));
			$return=array_merge($return, array_combine($replacements, $images));
		}

		if(!empty($cachefile)) {
			system_save($cachefile, array('emoticons'=>$return));
		}

		return $return;
	}

}

$GLOBALS['plugin_bbtoolbar']=new plugin_bbtoolbar();

if(!function_exists('array_combine')) {
	/**
	 * Creates an array by using one array for keys and another for its values
	 *
	 * @param array $keys: Array of keys to be used.
	 * @param array $values: Array of values to be used 
	 * @return array: Returns the combined array 
	 * @package PHP
	 * @since 5.0
	 */
	function array_combine($keys, $values) {
		if(count($keys)!=count($values)) {
			trigger_error('array_combine(): Both parameters should have an equal number of elements', E_USER_WARNING);
			return false;
		}

		if(!is_array($keys) || !is_array($values)) {
			trigger_error('array_combine() expects parameters to be arrays', E_USER_WARNING);
			return false;
		}

		$keys=array_values($keys);
		$values=array_values($values);

		$combined=array();

		foreach($keys as $id=>$key) {
			$combined[$key]=$values[$id];
		}

		return $combined;
	}
}

if(defined('MOD_ADMIN_PANEL')) {
	include dirname(__FILE__).'/admin.php';
}